-- control.lua
-- Nov 2016

-- Oarc's Separated Spawn Scenario
-- 
-- I wanted to create a scenario that allows you to spawn in separate locations
-- From there, I ended up adding a bunch of other minor/major features
-- 
-- Credit:
--  RSO mod to RSO author - Orzelek - I contacted him via the forum
--  Tags - Taken from WOGs scenario 
--  Event - Taken from WOGs scenario (looks like original source was 3Ra)
--  Rocket Silo - Taken from Frontier as an idea
--
-- Feel free to re-use anything you want. It would be nice to give me credit
-- if you can.
-- 
-- Follow server info on @_Oarc_


-- To keep the scenario more manageable I have done the following:
--      1. Keep all event calls in control.lua (here)
--      2. Put all config options in config.lua
--      3. Put mods into their own files where possible (RSO has multiple)


-- Generic Utility Includes
require("locale/oarc_utils")
require("locale/hakto_utils")
require("locale/rso/rso_control")
require("locale/frontier_silo")
require("locale/tag")

-- Main Configuration File
require("config")

-- Scenario Specific Includes
require("separate_spawns")
require("separate_spawns_guis")


--------------------------------------------------------------------------------
-- ALL EVENT HANLDERS ARE HERE IN ONE PLACE!
--------------------------------------------------------------------------------


----------------------------------------
-- On Init - only runs once the first 
--   time the game starts
----------------------------------------
script.on_init(function(event)
    global.version = version
    
    -- Configures the map settings for enemies
    -- This controls evolution growth factors and enemy expansion settings.
    ConfigureAlienStartingParams()
    
    if ENABLE_SEPARATE_SPAWNS then
        InitSpawnGlobalsAndForces()
    end
    
    if ENABLE_RANDOM_SILO_POSITION then
        SetRandomSiloPosition()
    else
        SetFixedSiloPosition()
    end
    
    if FRONTIER_ROCKET_SILO_MODE then
        ChartRocketSiloArea(game.forces[MAIN_FORCE])
    end
    
    if ENABLE_BLUEPRINT_STRING then
        bps_init()
    end
    
    if ENABLE_EFFICIENT_BOTS then
        EnableEfficientRobots(game.forces[MAIN_FORCE])
    end
    
    global.welcome_msg = WELCOME_MSG
    global.welcome_msg_title = WELCOME_MSG_TITLE
    
    CreateHostTable()
end)


----------------------------------------
-- Freeplay rocket launch info
-- Slightly modified for my purposes
----------------------------------------
script.on_event(defines.events.on_rocket_launched, function(event)
    if FRONTIER_ROCKET_SILO_MODE then
        RocketLaunchEvent(event)
    end
end)


----------------------------------------
-- Chunk Generation
----------------------------------------
script.on_event(defines.events.on_chunk_generated, function(event)
    if ENABLE_UNDECORATOR then
        UndecorateOnChunkGenerate(event)
    end
    
    if ENABLE_RSO then
        RSO_ChunkGenerated(event)
    end
    
    if FRONTIER_ROCKET_SILO_MODE then
        -- generate multiple silos
        GenerateRocketSiloChunk(event)
    end
    
    -- This MUST come after RSO generation!
    if ENABLE_SEPARATE_SPAWNS then
        SeparateSpawnsGenerateChunk(event)
    end
end)


----------------------------------------
-- Gui Click
----------------------------------------
script.on_event(defines.events.on_gui_click, function(event)
    if ENABLE_TAGS then
        TagGuiClick(event)
    end
    
    if ENABLE_SEPARATE_SPAWNS then
        WelcomeTextGuiClick(event)
        SpawnOptsGuiClick(event)
        SpawnCtrlGuiClick(event)
        SharedSpwnOptsGuiClick(event)
    end
    
    if ENABLE_BLUEPRINT_STRING then
        bps_on_gui_click(event)
    end
    
    if ENABLE_PLAYERLIST then
        PlayerListGuiClick(event)
    end
    
    if ENABLE_RULES then
        RuleGuiClick(event)
    end
end)


----------------------------------------
-- Player Events
----------------------------------------
script.on_event(defines.events.on_player_joined_game, function(event)
    local player = game.players[event.player_index]
    PlayerJoinedMessages(event)
    
    if ENABLE_TAGS then
        CreateTagGui(event)
    end
    
    if ENABLE_PLAYERLIST then
        CreatePlayerListGui(event)
    end
    
    if ENABLE_RULES then
        CreateRuleGui(event)
    end
end)

script.on_event(defines.events.on_player_created, function(event)
    -- log the first player (save creator) as host
    LogHost(game.players[event.player_index])

    SetHostServerMessages(event)
    
    if ENABLE_LONGREACH then
        GivePlayerLongReach(game.players[event.player_index])
    end
    
    if ENABLE_LARGERINVENTORY then
        GivePlayerLargerInventory(game.players[event.player_index])
    end
    
    if ENABLE_FASTERMOVEMENT then
        GivePlayerFasterMovement(game.players[event.player_index])
    end
    
    if not ENABLE_SEPARATE_SPAWNS then
        PlayerSpawnItems(event)
    else
        SeparateSpawnsPlayerCreated(event)
    end
    
end)

script.on_event(defines.events.on_player_died, function(event)
    if ENABLE_GRAVESTONE_CHESTS then
        CreateGravestoneChestsOnDeath(event)
    end
end)

script.on_event(defines.events.on_player_respawned, function(event)
    if not ENABLE_SEPARATE_SPAWNS then
        PlayerRespawnItems(event)
    else 
        SeparateSpawnsPlayerRespawned(event)
    end
    
    if ENABLE_LONGREACH then
        GivePlayerLongReach(game.players[event.player_index])
    end
    
    if ENABLE_LARGERINVENTORY then
        GivePlayerLargerInventory(game.players[event.player_index])
    end
    
    if ENABLE_FASTERMOVEMENT then
        GivePlayerFasterMovement(game.players[event.player_index])
    end
end)

script.on_event(defines.events.on_player_left_game, function(event)
    if ENABLE_SEPARATE_SPAWNS then
        FindUnusedSpawns(event)
    end
    
end)

script.on_event(defines.events.on_built_entity, function(event)
    if ENABLE_AUTOFILL then
        Autofill(event)
    end
end)

----------------------------------------
-- On Research Finished
----------------------------------------
script.on_event(defines.events.on_research_finished, function(event)
    if FRONTIER_ROCKET_SILO_MODE then
        RemoveRocketSiloRecipe(event)
    end
    
    if ENABLE_BLUEPRINT_STRING then
        bps_on_research_finished(event)
    end
    
    -- Example of how to remove a particular recipe:
    -- RemoveRecipe(event, "beacon")
end)

----------------------------------------
-- BPS Specific Event
----------------------------------------
script.on_event(defines.events.on_robot_built_entity, function(event)
    if ENABLE_BLUEPRINT_STRING then
        bps_on_robot_built_entity(event)
    end
end)

function printSpawns()
    for key,varx in pairs(global.uniqueSpawns) do
        for key,var in pairs(varx) do
            game.players["Hakto"].print(key .. "/" .. var)
        end
    end
end
